/*

*************************************************************************

ArmageTron -- Just another Tron Lightcycle Game in 3D.
Copyright (C) 2000  Manuel Moos (manuel@moosnet.de)
Copyright (C) 2003  Ben Hines <bhines@alumni.ucsd.edu>

**************************************************************************

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
***************************************************************************

*/
/*   SDLMain.m - main entry point for our Cocoa-ized SDL app
       Initial Version: Darrell Walisser <dwaliss1@purdue.edu>
       Non-NIB-Code & other changes: Max Horn <max@quendi.de>      
*/
#import <AppKit/AppKit.h>
#import <SDL/SDL.h>
#import "SDLMain.h"
#import <sys/param.h> /* for MAXPATHLEN */
#import <unistd.h>
#include <iostream>

#include "config.h"
#include "tError.h"
#include "tConfiguration.h"
#include "tDirectories.h"
#include "uMenu.h"

/* Use this flag to determine whether we use SDLMain.nib or not */
#define		SDL_USE_NIB_FILE	1
#define 	LASTVERSIONKEY		@"LastVersion"

static int    gArgc;
static char  **gArgv;
static BOOL   gFinderLaunch;
extern tString sn_programVersion; // our version
tString userDataPath;
tString varPath;

#if SDL_USE_NIB_FILE
/* A helper category for NSString */
@interface NSString (ReplaceSubString)
- (NSString *)stringByReplacingRange:(NSRange)aRange with:(NSString *)aString;
@end
#else
/* An internal Apple class used to setup Apple menus */
@interface NSAppleMenuController:NSObject {}
- (void)controlMenu:(NSMenu *)aMenu;
@end
#endif

SDL_Event event;
@interface SDLApplication : NSApplication
@end

@implementation SDLApplication
/* Invoked from the Quit menu item */

- (void)terminate:(id)sender
{
/* Post a SDL_QUIT event */
//    event.type = SDL_QUIT;
//    SDL_PushEvent(&event);
    
    st_SaveConfig();
    uMenu::quickexit=true;  
}

@end

//this changes the current working directory to the resource folder of 
//the .app bundle in Mac OS X
void MacOSX_SetCWD(char **argv) {
    char buffer[300];
    int count = 2, i;
    strcpy(buffer, argv[0]);
    
    if ( !strstr( buffer, ".app") ) {
        //it's not a .app bundle
        return;
    }

    for (i = strlen(buffer); i > 0 && count > 0; i--) {
        if (buffer[i] == '/')
            count--;
    }
    if (i == 0) 
        return;
    i+=2;
    buffer[i] = 0;
    strcat( buffer, "Resources");
    chdir( buffer );
}

/* The main class of the application, the application's delegate */
@implementation SDLMain

- (void)quit:(id)sender
{
// Why doesn't SDL_PushEvent work?
//    event.type = SDL_QUIT;
//    SDL_PushEvent(&event); 
    st_SaveConfig();
    uMenu::quickexit=true;      
}

/* Fix menu to contain the real app name instead of "SDL App" */
- (void)fixMenu:(NSMenu *)aMenu withAppName:(NSString *)appName
{
    NSRange aRange;
    NSEnumerator *enumerator;
    NSMenuItem *menuItem;

    aRange = [[aMenu title] rangeOfString:@"SDL App"];
    if (aRange.length != 0)
        [aMenu setTitle: [[aMenu title] stringByReplacingRange:aRange with:appName]];

    enumerator = [[aMenu itemArray] objectEnumerator];
    while ((menuItem = [enumerator nextObject]))
    {
        aRange = [[menuItem title] rangeOfString:@"SDL App"];
        if (aRange.length != 0)
            [menuItem setTitle: [[menuItem title] stringByReplacingRange:aRange with:appName]];
        if ([menuItem hasSubmenu])
            [self fixMenu:[menuItem submenu] withAppName:appName];
    }
    [ aMenu sizeToFit ];
}

/* Called when the internal event loop has just started running */
- (void) applicationDidFinishLaunching: (NSNotification *) note
{
    int status;
    NSString * varPathString;
    NSString * userDataPathString;
    NSArray *paths;
    /* Create cfg dir if it doesn't exist yet */
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSUserDefaults * defaults = [NSUserDefaults standardUserDefaults];
    BOOL isExists, isDirectory;
    NSString 	*lastVersion, *currentVersion;

    /* Find User's prefs folder and assign our var dir */
    paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    if (paths && [paths count] > 0) {
        varPath = [ [NSString stringWithString:
                            [[paths objectAtIndex:0] stringByAppendingPathComponent:@"Preferences/Armagetron"]] 
                            cString ];
        tDirectories::SetVar(varPath);
        varPathString = [NSString stringWithCString:varPath];
    } else
        varPathString = [[NSString stringWithCString:"~/Library/Preferences/Armagetron"] stringByExpandingTildeInPath];
    
    lastVersion = [defaults stringForKey:LASTVERSIONKEY];
    currentVersion = [NSString stringWithCString:sn_programVersion];
    if(!lastVersion)
    {
        /* Copy prefs from first OS X version to where they should go (if they are there) 
        (This was a hard coded path then. Thus, i use it again :)
        */
        isExists = [fileManager fileExistsAtPath:@"/Library/Application Support/Armagetron/user.cfg"];
        if (isExists) {
            if(![fileManager movePath:@"/Library/Application Support/Armagetron"
                        toPath:varPathString handler:nil])
            {           
                [fileManager copyPath:@"/Library/Application Support/Armagetron"
                            toPath:varPathString handler:nil];
                // If we fail, fine. They can reset the prefs.
            }
        }
        [defaults setObject:currentVersion forKey:LASTVERSIONKEY];
    } else {
        if([currentVersion intValue] > [lastVersion intValue])
            [defaults setObject:currentVersion forKey:LASTVERSIONKEY];
    }
    
    /* Create ours if needed */
    isExists = [fileManager fileExistsAtPath:varPathString isDirectory:&isDirectory];
    if (isExists & !isDirectory) {
            NSLog(@"path %@ already exists but not a directory", varPathString);
    } else if (!isExists) {
            if (![fileManager createDirectoryAtPath:varPathString attributes:nil]) {
                    NSLog(@"can't make directory %@", varPathString);
            }
    }
        
    /* See if we have user data anywhere */
 //   paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    if (paths && [paths count] > 0) {
	userDataPath = [[NSString stringWithString:[[paths objectAtIndex:0] 
                                    stringByAppendingPathComponent:@"Application Support/Armagetron"]] cString];
        tDirectories::SetUserData(userDataPath);
        userDataPathString = [NSString stringWithCString:userDataPath];
    }   else
        userDataPathString = [[NSString stringWithCString:"~/Library/Application Support/Armagetron"] stringByExpandingTildeInPath];
        
    /* Create ours if needed */
    isExists = [fileManager fileExistsAtPath:userDataPathString isDirectory:&isDirectory];
    if (isExists & !isDirectory) {
            NSLog(@"path %@ already exists but not a directory", userDataPathString);
    } else if (!isExists) {
            if (![fileManager createDirectoryAtPath:userDataPathString attributes:nil]) {
                    NSLog(@"can't make directory %@", userDataPathString);
            }
    }

#if SDL_USE_NIB_FILE
    /* Set the main menu to contain the real app name instead of "SDL App" */
    [self fixMenu:[NSApp mainMenu] withAppName:[[NSProcessInfo processInfo] processName]];
#endif

    /* Hand off to main application code */
    status = SDL_main (gArgc, gArgv);

    /* We're done, thank you for playing */
    exit(status);
}
@end


@implementation NSString (ReplaceSubString)

- (NSString *)stringByReplacingRange:(NSRange)aRange with:(NSString *)aString
{
    unsigned int bufferSize;
    unsigned int selfLen = [self length];
    unsigned int aStringLen = [aString length];
    unichar *buffer;
    NSRange localRange;
    NSString *result;

    bufferSize = selfLen + aStringLen - aRange.length;
    buffer = (unichar *) NSAllocateMemoryPages(bufferSize*sizeof(unichar));
    
    /* Get first part into buffer */
    localRange.location = 0;
    localRange.length = aRange.location;
    [self getCharacters:buffer range:localRange];
    
    /* Get middle part into buffer */
    localRange.location = 0;
    localRange.length = aStringLen;
    [aString getCharacters:(buffer+aRange.location) range:localRange];
     
    /* Get last part into buffer */
    localRange.location = aRange.location + aRange.length;
    localRange.length = selfLen - localRange.location;
    [self getCharacters:(buffer+aRange.location+aStringLen) range:localRange];
    
    /* Build output string */
    result = [NSString stringWithCharacters:buffer length:bufferSize];
    
    NSDeallocateMemoryPages(buffer, bufferSize);
    
    return result;
}

@end



#ifdef main
#  undef main
#endif


/* Main entry point to executable - should *not* be SDL_main! */
int main (int argc, const char **argv)
{

    /* Copy the arguments into a global variable */
    int i;
    
    /* This is passed if we are launched by double-clicking */
    if ( argc >= 2 && strncmp (argv[1], "-psn", 4) == 0 ) {
        gArgc = 1;
	gFinderLaunch = YES;
    } else {
        gArgc = argc;
	gFinderLaunch = NO;
    }
    gArgv = (char**) malloc (sizeof(*gArgv) * (gArgc+1));
    tASSERT (gArgv != NULL);
    for (i = 0; i < gArgc; i++)
        gArgv[i] = (char *) argv[i];
    gArgv[i] = NULL;
	
	MacOSX_SetCWD(gArgv);
#if SDL_USE_NIB_FILE
    [SDLApplication poseAsClass:[NSApplication class]];
    NSApplicationMain (argc, argv);
#else
    CustomApplicationMain (argc, argv);
#endif
    return 0;
}
